/* GTK+ - The GIMP ToolKit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * BeOS Port
 * Copyright (C) 1999 EventLoop, Inc.
 *   Shawn T. Amundson <amundson@gtk.org>
 *   James Mitchell <mitchell@eventloop.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
 */


#include "gtk_bapp.h"
#include <iostream.h>
#include <gtk/gtk.h>
#include <kernel/OS.h>

extern "C" {
void gtk_bapp_init ();
gboolean gtk_bapp_run ();
void gtk_bapp_quit ();
}

GtkBApp::GtkBApp(const char *signature)
  : GdkBApp(signature)
{
  run = false;		
}

void
GtkBApp::Pulse()
{
  while (g_main_iteration(FALSE)) ;
}

void
GtkBApp::SetRun(bool state)
{
  run = state;
}

bool
GtkBApp::NeedtoRun()
{
  if (run)
    return false;
  return true;
}

/* david - 19th july 2000
 * removed the call to gtk_main_quit that I had here as it was generating
 * an infinite loop!!  doh!
 */
bool
GtkBApp::QuitRequested(void)
{
  g_print("GtkBApp::QuitRequested\n");
}

void
gtk_bapp_init ()
{
  g_print("gtk_bapp_init\n");
  if (!be_app)
    new GtkBApp(GTK_APP_SIGNATURE);
}

/* david 19th July 2000
 * This should only ever be called once, calling it more often
 * is an error.  The check of NeedtoRun() was added to stop the error
 * and hopefully allow us to find out why it was happening.
 */

gboolean
gtk_bapp_run ()
{
  if (((GtkBApp*)be_app)->NeedtoRun()) {
    be_app->SetPulseRate(100000);
    ((GtkBApp*)be_app)->SetRun(true);
    g_print("gtk_bapp_run\n");
    be_app->Run();
    g_print("gtk_bapp_run - Run() has returned!\n");
  }
  /* we get here because either...
   * - we've returned from Run() in which case we're about to quit
   * - we've got here becuase we've been called when we've already 
   *   run.
   * In either case we want to signal to the glib main loop that we don't 
   * need to be called again, which we do by returning FALSE.
   */
  return FALSE;
}


void
gtk_bapp_quit ()
{
  g_print("gtk_bapp_quit!\n");
  be_app->PostMessage(B_QUIT_REQUESTED);
}
